%% PROBECONTACTLIB
%% EstimateAperture - Estimate effective aperture based on lateral power spectrum
% Input:
%   data - beam data (autocorr estimator) or beam power spectrum (moment estimator, linear scaled)
%   p.estAlg - mean frequency and bandwidth estimator algorithm ['moment', 'autocorr']
%   p.txAperture / p.rxAperture - width of transmit and receive aperture
%   p.txApodization / p.rxApodization - apodization of transmit and receive aperture
%
% Output:
%   out.apCenter - Estimated center position of effective aperture
%   out.apWidth - Estimated width of effective aperture
%
function out = EstimateAperture(data, dataAxis, p)
%% Default setup
    p = struct();
    p.estAlg = 'moment';        % ['moment', 'autocorr']
    p.apEstAlg = 'mean';        % ['mean', 'bandwidth']
   
    p.txAperture = 0.022;       % Tx aperture dimensions
    p.rxAperture = 0.022;       % Rx aperture dimensions
    p.txApodization = 'rect';   % Tx apodization type
    p.rxApodization = 'rect';   % Rx apodization type
    
%% Override with provided settings
    if(exist('p'))
        fields = fieldnames(p);
        for k=1:size(fields,1);
            if(isfield(p,fields{k}))
                p = setfield(p, fields{k}, getfield(p,fields{k}));
            else
                disp(['EstimateAperture: ' fields{k} ' is not a valid parameter...']);
            end
        end
    end

%% Estimate bandwidth of a unit theoretical aperture
        [convAp, convAxis, apExtent] = calcConvAperture(p, scaleFactor*txAperture, scaleFactor*rxAperture);        
        % Normalize to unit aperture extent
        unitConvAp = convAp / apExtent; unitConvAxis = convAxis / apExtent;
        % 1st and 2nd moment estimation
        x_mean_u = unitConvAp.'*.unitConvAxis.'/sum(unitConvAp); % First moment (center)                  
        axScale = 1;
        x_bw_u = sqrt(unitConvAp.'*(unitConvAxis.' - x_mean_u).^2/sum(unitConvAp));
                
%% Estimate aperture
        offsetLims = out.apDim/4;
        I1 = find(out.apAxis < -(out.apDim/2 + offsetLims));
        I2 = find(out.apAxis > (out.apDim/2 + offsetLims));
        startI = max(1,I1(end)); endI = I2(1);

        if(strcmpi(p.estAlg,'moment'))
            % 1st and 2nd moment estimation
            out.x_mean = out.latSpect(startI:endI).'*out.apAxis(startI:endI).'/sum(out.latSpect(startI:endI)); % First moment (center)                  
            out.x_bw = 2*axScale*sqrt(out.latSpect(startI:endI).'*(out.apAxis(startI:endI).' - out.x_mean).^2/sum(out.latSpect(startI:endI)));
        elseif(strcmpi(p.estAlg,'autocorr'))
            % Autocorrelation algorithm
            R0Est = mean(conj(out.latMap).*out.latMap);
            R1Est = mean(conj(out.latMap(1:end-1)).*out.latMap(2:end));
            out.x_mean = axScale*lambda*info.focusPos*angle(R1Est) / (2*pi*info.latInc*info.focusPos);   
            out.x_bw = lambda*info.focusPos*sqrt(2)*sqrt(1 - abs(R1Est)/R0Est);
        else
            disp(['EstimateAperture: ' p.estAlg ' is not a valid estimation algorithm...']);
            return;
        end    
        
        if(strcmpi(p.apEstAlg,'mean'))
            scaleFactor = (1-abs(out.x_mean)/(apExtent/2));
        elseif(strcmpi(p.apEstAlg,'bandwidth'))

        else
            disp(['EstimateAperture: ' p.apEstAlg ' is not a valid aperture estimation algorithm...']);
            return;
        end    
        [out.EstConvAp, out.EstConvAxis, out.EstApExtent] = calcConvAperture(p, scaleFactor*txAperture, scaleFactor*rxAperture);        
        out.EstConvAxis = out.EstConvAxis*axScale;
        out.EstApFactor = scaleFactor;
        
end % End function