function out = calcProbeContact(IQ, setup, info)

%% Default setup
    p = struct();
    p.curFrame = 1;
    p.avgRange = 0.03;          % [m] averaging region centered around focus point
    p.avgSector = info.width*180/pi;
    p.NFFT = 512;
    p.spectWin = 'hamming';                                 % blir brukt!
    p.NSmooth = 1;
    p.clutterFilter = 0;        % Do clutter filtering before contact estimation
    p.harmonic = 0;             % Harmonic imaging (tx x tx x rx)
    
    p.explicitApertures = 0;    % Use explicit apertures
    p.txAperture = 0.022;       % Tx aperture dimensions (if explicitApertures=1)
    p.rxAperture = 0.022;       % Rx aperture dimensions (if explicitApertures=1)
    p.txApodization = 'rect';   % Tx apodization type (if explicitApertures=1)
    p.rxApodization = 'rect';   % Rx apodization type (if explicitApertures=1)
    
    p.plotTwowayAperture = 0;   % Plot the twoway aperture
    p.scaling = 'twoway';
    p.estimateAperture = 1;
    p.gain = 0;
    out = struct();
    
%% Override with provided settings
    if(exist('setup'))
        fields = fieldnames(setup);
        for k=1:size(fields,1);
            if(isfield(p,fields{k}))
                p = setfield(p, fields{k}, getfield(setup,fields{k}));
            else
                disp(['calcProbeContact: ' fields{k} ' is not a valid parameter...']);
            end
        end
    end
                  
%% Do clutter filtering if set
    if(info.packetsize > 1)
       if(p.clutterFilter)                      
           %b = [1 -1];
           Fm = polyreg(info.packetsize,1,0,1,20);
           IQFilt = reshape(IQ,[info.packetsize,info.ranges*info.beams]);
           IQFilt = Fm*IQFilt;
           IQFilt = reshape(IQFilt,[size(IQFilt,1),info.ranges,info.beams]);           
           IQ = squeeze(IQFilt(1,:,:)); 
       else
           IQ = squeeze(mean(IQ,1));
       end
    end
    
%% Calculate lateral map (average range samples)
    startSample = round((info.focusPos - info.depthStart - p.avgRange/2)/info.depthInc);
    endSample   = round((info.focusPos - info.depthStart + p.avgRange/2)/info.depthInc);      
    startBeam   = max(1, round((info.width/2-p.avgSector/2*pi/180)/info.latInc));
    endBeam     = min(info.beams, round((info.width/2+p.avgSector/2*pi/180)/info.latInc));
    out.latMap  = mean(IQ(startSample:endSample,startBeam:endBeam)).';
      
%% Calculate lateral spectrum
    if(strcmpi(p.spectWin,'rectangular'))
        win = boxcar(length(out.latMap));
    elseif(strcmpi(p.spectWin,'hamming'))                    
        win = hamming(length(out.latMap));
    elseif(strcmpi(p.spectWin,'hanning'))
        win = hanning(length(out.latMap));    
    end

    % Remove DC component
    out.latMap = out.latMap - mean(out.latMap);

    % Calculate power spectrums
    [latSpect,w] = pwelch(out.latMap); 
    %[latSpect,w] = pwelch(out.latMap,[],[],[],[],'twosided');
    latSpect = fftshift(2*pi*latSpect); % Power spectrum                   
    
%% Smooth spectrum
    b = ones(p.NSmooth,1)/sqrt(p.NSmooth);
    out.latSpect = filter2(b,latSpect);

%% Calculate contact map    
    out.contactMap = max(0,10*log10(abs(out.latSpect)+eps) + p.gain); 
    out.params = p;  
       
%% Calculate theoretical output aperture
    if(p.explicitApertures)
        if(p.harmonic)
            txAperture = p.txAperture/2; % Rx f_demod is reference, scale tx aperture by a factor 1/2
        else
            txAperture = p.txAperture;            
        end
        rxAperture = p.rxAperture;        
        [out.convAp, out.convAxis, apExtent] = calcConvAperture(p, txAperture, rxAperture);            
    else
        txAperture = info.txAperture;
        apExtent = 2*txAperture;        % Does txAperture vary in UDT?
    end
        
    % Make aperture axis
    lambda = 1540/info.fDem;
    D = lambda / info.latInc;
    axScale = 1;
    if(strcmp(p.scaling,'oneway'))
        if(p.harmonic)
            axScale = 2*txAperture/apExtent;
            apExtent = 2*txAperture;
        else
            axScale = txAperture/apExtent;
            apExtent = txAperture;
        end
        D = D*axScale;
        out.convAxis = out.convAxis*axScale;
    end
        
    out.apAxis = linspace(-D/2,D/2,length(out.contactMap));
    out.apDim = apExtent;
    
%% Estimate aperture
    if(p.estimateAperture)
        offsetLims = out.apDim/4;
        I1 = find(out.apAxis < -(out.apDim/2 + offsetLims));
        I2 = find(out.apAxis > (out.apDim/2 + offsetLims));
        startI = max(1,I1(end)); endI = I2(1);          

        % Mean frequency estimation
        out.x_mean = out.latSpect(startI:endI).'*out.apAxis(startI:endI).'/sum(out.latSpect(startI:endI)); % First moment (center)          
        R0Est = mean(conj(out.latMap).*out.latMap);                                                         % autokorr. metoden
        R1Est = mean(conj(out.latMap(1:end-1)).*out.latMap(2:end));
        out.x_meanAcorr = axScale*lambda*info.focusPos*angle(R1Est) / (2*pi*info.latInc*info.focusPos);     % [m]
        
        % Bandwidth estimation
        out.x_bw = 2*axScale*sqrt(out.latSpect(startI:endI).'*(out.apAxis(startI:endI).' - out.x_mean).^2/sum(out.latSpect(startI:endI)));
        %out.x_bwAcorr = lambda*info.focusPos*sqrt(2)*sqrt(1 - abs(R1Est)/R0Est);
        %out.x_bwAcorr = lambda*info.focusPos*sqrt(2)*sqrt(1 - abs(R1Est)/R0Est)/(info.latInc*info.focusPos); % [m]
        
        scaleFactor = (1-abs(out.x_mean)/(apExtent/2));
        %scaleFactor = (1-abs(out.x_meanAcorr)/(apExtent/2));
        [out.EstConvAp, out.EstConvAxis, out.EstApExtent] = calcConvAperture(p, scaleFactor*txAperture, scaleFactor*rxAperture);        
        out.EstConvAxis = out.EstConvAxis*axScale;
        out.EstApFactor = scaleFactor;
    end

%% Added by Cecilie ins Simpson, 06.06.09

    normDFT = 1;    %use normalized DFT

    % Testing for 4 cases
    N               = endSample - startSample + 1;

    % A: averaging in 1D -> remove dc -> pwelch -> smooth -> log comp
    map_A           = mean(IQ(startSample:endSample,startBeam:endBeam),1);
    map_A           = map_A - mean(map_A);
    [spect_A,w]     = pwelch(map_A); 
    spect_A         = fftshift(2*pi*spect_A); % Power spectrum
    b               = ones(p.NSmooth,1)/sqrt(p.NSmooth);
    spect_A         = filter2(b,spect_A);
    contactMap_A    = max(0,10*log10(abs(spect_A)+eps) + p.gain);

    % B: averaging in 1D -> remove dc -> windowing -> 1 D FFT -> abs + ^2 -> smooth -> log comp
    map_B           = mean(IQ(startSample:endSample,startBeam:endBeam),1);
    map_B           = map_B - mean(map_B);
    win             = hamming(length(map_B));      
    scale           = sum(win(:))/length(map_B);
    spect_B         = abs(fftshift(fft(win'.*map_B,256))).^2;
    spect_B         = 1/scale*spect_B;
    if (normDFT)
        spect_B     = (1/length(map_B))*spect_B;
    end
    b               = ones(p.NSmooth,1)/sqrt(p.NSmooth);
    spect_B         = filter2(b',spect_B);
    contactMap_B    = max(0,10*log10(abs(spect_B)+eps) + p.gain);

    % C: Remove dc -> windowing -> 1D FFT along rows-> abs + ^2 -> averaging -> smoothing -> log comp
    map_C           = IQ(startSample:endSample,startBeam:endBeam);
    map_C           = map_C - mean2(map_C);   
    win             = boxcar(size(map_C,1))*hamming(size(map_C,2))';
    scale           = sum(win(:))/(size(map_C,1)*size(map_C,2));
    spect_C         = abs(fftshift(fft(win.*map_C,256,2))).^2;
    spect_C         = 1/scale*spect_C;
    if (normDFT)
        spect_C     = (1/(size(map_C,2)))*spect_C;
    end   
    spect_C         = mean(spect_C,1);
    spect_C2        = spect_C*N;
    b               = ones(p.NSmooth,1)/sqrt(p.NSmooth);        
    spect_C         = filter2(b',spect_C);
    contactMap_C    = max(0,10*log10(abs(spect_C)+eps) + p.gain);
    spect_C2        = filter2(b',spect_C2);
    contactMap_C2   = max(0,10*log10(abs(spect_C2)+eps) + p.gain);

    % D: Remove dc -> windowing -> 2D FFT -> averaging -> smoothing -> log comp
    map_D           = IQ(startSample:endSample,startBeam:endBeam);
    map_D           = map_D - mean2(map_D);
    win             = boxcar(size(map_D,1))*hamming(size(map_D,2))';
    scale           = sum(win(:))/(size(map_D,1)*size(map_D,2));
    spect_D         = abs(fftshift(fft2(win.*map_D,size(map_D,1),256))).^2;
    spect_D         = 1/scale*spect_D;
    if (normDFT)
        spect_D     = (1/(size(map_D,1)*size(map_D,2)))*spect_D;
    end
    spect_D         = mean(spect_D,1);
    b               = ones(p.NSmooth,1)/sqrt(p.NSmooth);        
    spect_D         = filter2(b',spect_D);
    contactMap_D    = max(0,10*log10(abs(spect_D)+eps) + p.gain);

    %2D
    win             = hamming(size(map_D,1))*hamming(size(map_D,2))';
    scale           = sum(win(:))/(size(map_D,1)*size(map_D,2));
    spect_2D        = abs(fftshift(fft2(win.*map_D,size(map_D,1),256))).^2;
    spect_2D        = 1/scale*spect_2D;
    if (normDFT)
        spect_2D    = (1/(size(map_D,1)*size(map_D,2)))*spect_2D;
    end
    b               = ones(p.NSmooth,p.NSmooth)/p.NSmooth;
    spect_2D        = filter2(b,spect_2D);
    spectLat        = mean(spect_2D,1);
    spectRad        = mean(spect_2D,2);
    logSpect_2D     = max(0,10*log10(abs(spect_2D)+eps) + p.gain);
    
    
    if(p.estimateAperture)    
        % xaxis
        fx          = 1/(info.focusPos*info.latInc);
        kx          = (2*pi*fx)/1000;       % [rad/mm]
        kxaxis      = linspace(-kx/2,kx/2,size(spect_2D,2));
        % yaxis
        f           = linspace(-info.frs/2,info.frs/2,size(spect_2D,1));
        freq        = f/1e6;                % Frequency axis, MHz
        
        % Calculations of mean frequecy and bandwidth 
        offsetLims  = out.apDim/4;
        
        lat_axis    = linspace(-D/2,D/2,length(spect_A));
        I1          = find(lat_axis < -(out.apDim/2 + offsetLims));
        I2          = find(lat_axis > (out.apDim/2 + offsetLims));
        startI      = max(1,I1(end)); endI = I2(1);  
                
        % CASE 1
        lat_mean_A  = spect_A(startI:endI).'*lat_axis(startI:endI).'/sum(spect_A(startI:endI));
        lat_bw_A    = 2*axScale*sqrt(spect_A(startI:endI).'*(lat_axis(startI:endI).' - lat_mean_A).^2/sum(spect_A(startI:endI)));
        lat_mean_A  = lat_mean_A*1000; %[mm]
        % CASE 2
        spect_B     = spect_B'; 
        lat_mean_B  = spect_B(startI:endI).'*lat_axis(startI:endI).'/sum(spect_B(startI:endI));
        lat_bw_B    = 2*axScale*sqrt(spect_B(startI:endI).'*(lat_axis(startI:endI).' - lat_mean_B).^2/sum(spect_B(startI:endI)));
        lat_mean_B  = lat_mean_B*1000; %[mm]
        % CASE 3
        spect_C     = spect_C';
        lat_mean_C  = spect_C(startI:endI).'*lat_axis(startI:endI).'/sum(spect_C(startI:endI));
        lat_bw_C    = 2*axScale*sqrt(spect_C(startI:endI).'*(lat_axis(startI:endI).' - lat_mean_C).^2/sum(spect_C(startI:endI)));
        lat_mean_C  = lat_mean_C*1000; %[mm]
        % CASE 4
        spect_D     = spect_D';  
        lat_mean_D  = spect_D(startI:endI).'*lat_axis(startI:endI).'/sum(spect_D(startI:endI));
        lat_bw_D    = 2*axScale*sqrt(spect_D(startI:endI).'*(lat_axis(startI:endI).' - lat_mean_D).^2/sum(spect_D(startI:endI)));
        lat_mean_D  = lat_mean_D*1000; %[mm]
        spectLat = spectLat';
        contactLat  = max(0,10*log10(abs(spectLat)+eps) + p.gain);
        contactRad  = max(0,10*log10(abs(spectRad)+eps) + p.gain);

        % Lateral
        lat_axis    = linspace(-D/2,D/2,length(spectLat));
        I1          = find(lat_axis < -(out.apDim/2 + offsetLims));
        I2          = find(lat_axis > (out.apDim/2 + offsetLims));
        startI      = max(1,I1(end)); endI = I2(1);     
        lat_mean    = spectLat(startI:endI).'*lat_axis(startI:endI).'/sum(spectLat(startI:endI));
        lat_bw      = 2*axScale*sqrt(spectLat(startI:endI).'*(lat_axis(startI:endI).' - lat_mean).^2/sum(spectLat(startI:endI)));

        % Radial
        z           = (0:size(IQ,1)-1)/size(IQ,1)*info.depthEnd;
        zDiff       = z(endSample) - z(startSample);
        rad_axis    = linspace(-zDiff/2,zDiff/2,length(spectRad));
        rad_mean    = spectRad.'*rad_axis.'/sum(spectRad);
        rad_bw      = 2*axScale*sqrt(spectRad.'*(rad_axis.' - rad_mean).^2/sum(spectRad));

        % Plot of spectrum
        figure;
        % Radial spectrum
        sp1 = subplot(2,2,1);
        plot(contactRad,rad_axis*100,'LineWidth',1.5);
        hold on;
        plot(get(gca,'XLim'),[rad_mean rad_mean]*100,'g--','LineWidth',1.5);
        YBw = max(contactRad) - 10*log10(exp(3^2*0.5)); 
        plot([max(contactRad)-3 max(contactRad)-3],[rad_mean-rad_bw/2, rad_mean+rad_bw/2]*100,'m','LineWidth',1.5);
        hold off;
        title('Radial spectrum')
        ylabel('[cm]')
        xlabel('Power [dB]')
        axis([min(contactRad)-3 max(contactRad)+3 -0.5 0.5])
        % 2D spectrum
        sp2 = subplot(2,2,2);
        imagesc(kxaxis,freq,logSpect_2D);
        title('2D Spectrum')
        xlabel('k_x')
        ylabel('Frequency [MHz]')
        colormap(gray)
        % Lateral spectrum
        sp4 = subplot(2,2,4);
        pl1 = plot(lat_axis*100,contactLat,'LineWidth',1.5);
        hold on;
        pl2 = plot([lat_mean lat_mean]*100,get(gca,'YLim'),'g--','LineWidth',1.5);
        YBw = max(contactLat) - 10*log10(exp(3^2*0.5));  
        pl3 = plot([lat_mean-lat_bw/2, lat_mean+lat_bw/2]*100,[max(contactLat)-3 max(contactLat)-3],'m','LineWidth',1.5);
        hold off;
        title('Lateral spectrum')
        xlabel('[cm]')
        ylabel('Power [dB]')
        axis([-5 5 min(contactLat)-3 max(contactLat)+3])
        % Box
        sp3 = subplot(2,2,3);
        legend(sp3,[pl1;pl2;pl3],'Spectrum','Mean frequency','Bandwidth');
        axis(sp3,'off');
        % Rearrange
        p1 = get(sp1,'position');
        set(sp1,'position',[p1(1,1) p1(1,2)-0.15 p1(1,3)-0.1 p1(1,4)+0.15]);
        p2 = get(sp2,'position');
        set(sp2,'position',[p2(1,1)-0.1 p2(1,2)-0.15 p2(1,3)+0.1 p2(1,4)+0.15]);
        p3 = get(sp3,'position');
        set(sp3,'position',[p3(1)-0.05 p3(2)-0.15 p3(3)-0.05 p3(4)]);
        p4 = get(sp4,'position');
        set(sp4,'position',[p4(1,1)-0.1 p4(1,2) p4(1,3)+0.1 p4(1,4)-0.15]);       
    end
       
    figure;
    hold on;
    plot(lat_axis*100,contactMap_A,'r','LineWidth',1.5)
    plot(lat_axis*100,contactMap_B,'g','LineWidth',1.5)
    plot(lat_axis*100,contactMap_C,'k','LineWidth',1.5)
    plot(lat_axis*100,contactMap_C2,'m','LineWidth',1.5)
    plot(lat_axis*100,contactMap_D,'c--','LineWidth',1.5)
    hold off;
    if (normDFT)
        legend('Case 1','Case 2','Case 3','Case 3*N','Case 4')
    else
        legend('Case 1','Case 2','Case 3','Case 4')
    end
    ylabel('Power [dB]')
    xlabel('Lateral axis [cm]')
    if (normDFT)
        title('Lateral power spectrum, normalized Fourier transform')
    else
        title('Lateral power spectrum')
    end
        
end
